@@ -221,13 +221,13 @@ int fors_end(const cpl_frameset *frames, cpl_errorstate before_exec)
     if (cpl_error_get_code() == CPL_ERROR_NONE) {
         
         const cpl_frame *f;
+        int i_frame = 0;
         
         cpl_msg_info(cpl_func, "Product frame%s:", 
                      cpl_frameset_get_size(frames) != 1 ? "s" : "");
 
-        for (f = cpl_frameset_get_first_const(frames);
-             f != NULL;
-             f = cpl_frameset_get_next_const(frames)) {
+        for (i_frame = 0; i_frame < cpl_frameset_get_size(frames); ++i_frame) {
+            f = cpl_frameset_get_position_const(frames, i_frame);
             if (cpl_frame_get_group(f) == CPL_FRAME_GROUP_PRODUCT) {
                 fors_frame_print(f);
             }
@@ -257,12 +257,13 @@ void
 fors_dfs_set_groups(cpl_frameset * set)
 {
     cpl_frame *f;
+    int i_frame = 0;
     
     assure( set != NULL, return, NULL );
     
-    for (f = cpl_frameset_get_first(set);
-         f != NULL;
-         f = cpl_frameset_get_next(set)) {
+    for (i_frame = 0; i_frame < cpl_frameset_get_size(set); ++i_frame) {
+
+        f = cpl_frameset_get_position_const(set, i_frame);
 
         const char *tag = cpl_frame_get_tag(f);
 
