@@ -125,8 +125,10 @@ test_bias(void)
             cpl_frameset_find(frames, BIAS));
         
         /* Verify that relative error decreased  */
+        /* 200 is the simulated overscan value. The master bias no longer
+         * has overscan */
         test( fors_image_get_error_mean(master_bias, NULL) /
-              fors_image_get_mean(master_bias, NULL) 
+              (fors_image_get_mean(master_bias, NULL) + 200)
               <
               fors_image_get_error_mean(raw_bias, NULL) /
               fors_image_get_mean(raw_bias, NULL));
