@@ -287,6 +287,14 @@ hdrl_test_mul(void)
 
         cpl_test_rel(a, 1.7 * 1.7, HDRL_EPS_DATA);
         cpl_test_rel(ea, 9.52, HDRL_EPS_ERROR * 10);
+
+        a = -1.7;
+        ea = 2.8;
+
+        hdrl_elemop_mul(&a, &ea, 1, &a, &ea, 1, NULL);
+
+        cpl_test_rel(a, 1.7 * 1.7, HDRL_EPS_DATA);
+        cpl_test_rel(ea, 9.52, HDRL_EPS_ERROR * 10);
     }
 
     {
@@ -407,6 +415,29 @@ hdrl_test_div(void)
         cpl_test_rel(ea[1], 0.3004626062886658, HDRL_EPS_ERROR * 10);
     }
 
+    /* division by 0. */
+    {
+        hdrl_data_t a[] = {2, 3};
+        hdrl_error_t ea[] = {0.5, 1};
+        hdrl_data_t b[] = {-2, 0.};
+        const hdrl_error_t eb[] = {0.5, 3};
+
+        hdrl_elemop_div(a, ea, ARRAY_LEN(a), b, eb, ARRAY_LEN(b), NULL);
+
+        cpl_test_rel(a[0], -1., HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], 0.35355339059327379, HDRL_EPS_ERROR * 10);
+        cpl_test(isnan(a[1]));
+        cpl_test(isnan(ea[1]));
+
+        b[0] = 0.;
+        hdrl_elemop_div(a, ea, ARRAY_LEN(a), b, eb, 1, NULL);
+
+        cpl_test(isnan(a[0]));
+        cpl_test(isnan(ea[0]));
+        cpl_test(isnan(a[1]));
+        cpl_test(isnan(ea[1]));
+    }
+
     {
         hdrl_data_t a[] = {2, 3};
         hdrl_error_t ea[] = {0.5, 1};
@@ -439,6 +470,223 @@ hdrl_test_div(void)
     return cpl_error_get_code();
 }
 
+static cpl_error_code
+hdrl_test_pow(void)
+{
+    {
+        hdrl_data_t a, b;
+        hdrl_error_t ea, eb;
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 2, NULL);
+        cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+    }
+
+    {
+        hdrl_data_t a = 1.2;
+        hdrl_error_t ea = 0.5;
+
+        hdrl_elemop_pow(&a, &ea, 1, &a, &ea, 1, NULL);
+
+        cpl_test_rel(a, pow(1.2, 1.2), HDRL_EPS_DATA);
+        cpl_test_rel(ea, 0.7357378647225408, HDRL_EPS_ERROR * 10);
+    }
+
+    /* square root test */
+    {
+        hdrl_data_t a = 1.2;
+        hdrl_error_t ea = 0.6;
+        hdrl_data_t b = 0.5;
+        hdrl_error_t eb = 0.;
+
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        cpl_test_rel(a, sqrt(1.2), HDRL_EPS_DATA);
+        cpl_test_rel(ea, 0.27386127875258304, HDRL_EPS_ERROR * 10);
+
+        a = -1.2;
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        cpl_test(isnan(a));
+        cpl_test(isnan(ea));
+
+        /* negative value test */
+        a = -1.2;
+        ea = 0.6;
+        b = 3.;
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        cpl_test_rel(a, pow(-1.2, 3.), HDRL_EPS_DATA);
+        cpl_test_rel(ea, 2.592, HDRL_EPS_ERROR * 10);
+    }
+
+    /* pow with error free power 2 exponent == repeated multiplication */
+    {
+        hdrl_data_t a = -1.2;
+        hdrl_error_t ea = 0.5;
+        hdrl_data_t b = 2.0;
+        hdrl_error_t eb = 0.0;
+
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        hdrl_data_t a2 = -1.2;
+        hdrl_error_t ea2 = 0.5;
+        hdrl_elemop_mul(&a2, &ea2, 1, &a2, &ea2, 1, NULL);
+
+        cpl_test_rel(a, a2, HDRL_EPS_DATA);
+        cpl_test_rel(ea, ea2, HDRL_EPS_ERROR * 10);
+
+        a = -1.2;
+        ea = 0.5;
+        b = 4.0;
+        eb = 0.0;
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+        hdrl_elemop_mul(&a2, &ea2, 1, &a2, &ea2, 1, NULL);
+        cpl_test_rel(a, a2, HDRL_EPS_DATA);
+        cpl_test_rel(ea, ea2, HDRL_EPS_ERROR * 10);
+    }
+    /* array pow with error free power 2 exponent == repeated multiplication */
+    {
+        hdrl_data_t a[] = {0.3, 10.};
+        hdrl_error_t ea[] = {0.5, 2.};
+        hdrl_data_t b[] = {2.0, 2.0};
+        hdrl_error_t eb[] = {0., 0.};
+
+        hdrl_elemop_pow(a, ea, 2, b, eb, 2, NULL);
+
+        hdrl_data_t a2[] = {0.3, 10.};
+        hdrl_error_t ea2[] = {0.5, 2.};
+        hdrl_elemop_mul(a2, ea2, 2, a2, ea2, 2, NULL);
+
+        cpl_test_rel(a[0], a2[0], HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], ea2[0], HDRL_EPS_ERROR * 10);
+        cpl_test_rel(a[1], a2[1], HDRL_EPS_DATA);
+        cpl_test_rel(ea[1], ea2[1], HDRL_EPS_ERROR * 10);
+    }
+
+    /* pow with error free -1 exponent == reciprocal */
+    {
+        hdrl_data_t a = 1.2;
+        hdrl_error_t ea = 0.5;
+        hdrl_data_t b = -1.0;
+        hdrl_error_t eb = 0.0;
+
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        hdrl_data_t a2 = 1.0;
+        hdrl_error_t ea2 = 0.;
+        hdrl_data_t b2 = 1.2;
+        hdrl_error_t eb2 = 0.5;
+        hdrl_elemop_div(&a2, &ea2, 1, &b2, &eb2, 1, NULL);
+
+        cpl_test_rel(a, a2, HDRL_EPS_DATA);
+        cpl_test_rel(ea, ea2, HDRL_EPS_ERROR * 10);
+    }
+
+    /* array pow with error free -1 exponent == reciprocal*/
+    {
+        hdrl_data_t a[] = {0.3, 10.};
+        hdrl_error_t ea[] = {0.5, 2.};
+        hdrl_data_t b[] = {-1.0, -1.0};
+        hdrl_error_t eb[] = {0., 0.};
+
+        hdrl_elemop_pow(a, ea, 2, b, eb, 2, NULL);
+
+        hdrl_data_t a2[] = {1.0, 1.0};
+        hdrl_error_t ea2[] = {0., 0.};
+        hdrl_data_t b2[] = {0.3, 10.};
+        hdrl_error_t eb2[] = {0.5, 2.};
+        hdrl_elemop_div(a2, ea2, 2, b2, eb2, 2, NULL);
+
+        cpl_test_rel(a[0], a2[0], HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], ea2[0], HDRL_EPS_ERROR * 10);
+        cpl_test_rel(a[1], a2[1], HDRL_EPS_DATA);
+        cpl_test_rel(ea[1], ea2[1], HDRL_EPS_ERROR * 10);
+    }
+
+    {
+        hdrl_data_t a = 1.2;
+        hdrl_error_t ea = 0.5;
+        hdrl_data_t b = 2.0;
+        hdrl_error_t eb = 2.0;
+
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        cpl_test_rel(a, pow(1.2, b), HDRL_EPS_DATA);
+        cpl_test_rel(ea, 1.3098531960320208, HDRL_EPS_ERROR * 10);
+    }
+
+    {
+        hdrl_data_t a = 0.;
+        hdrl_error_t ea = 0.5;
+        hdrl_data_t b = -1.;
+        hdrl_error_t eb = 0.;
+
+        hdrl_elemop_pow(&a, &ea, 1, &b, &eb, 1, NULL);
+
+        cpl_test(isnan(a));
+        cpl_test(isnan(ea));
+    }
+
+    {
+        hdrl_data_t a[] = {0., 0.};
+        hdrl_error_t ea[] = {0.5, 0.5};
+        hdrl_data_t b = -1.;
+        hdrl_error_t eb = 0.;
+
+        hdrl_elemop_pow(a, ea, 2, &b, &eb, 1, NULL);
+
+        cpl_test(isnan(a[0]));
+        cpl_test(isnan(ea[0]));
+        cpl_test(isnan(a[1]));
+        cpl_test(isnan(ea[1]));
+    }
+
+    {
+        hdrl_data_t a[] = {2, 3};
+        hdrl_error_t ea[] = {0.5, 1};
+        const hdrl_data_t b[] = {-2, 6};
+        const hdrl_error_t eb[] = {0.5, 3};
+
+        hdrl_elemop_pow(a, ea, ARRAY_LEN(a), b, eb, ARRAY_LEN(b), NULL);
+
+        cpl_test_rel(a[0], 0.25, HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], 0.15209233492346647, HDRL_EPS_ERROR * 10);
+        cpl_test_rel(a[1], 729, HDRL_EPS_DATA);
+        cpl_test_rel(ea[1], 2810.438304633068, HDRL_EPS_ERROR * 10);
+    }
+
+    {
+        hdrl_data_t a[] = {2, 3};
+        hdrl_error_t ea[] = {0.5, 1};
+        const hdrl_data_t b = -2;
+        const hdrl_error_t eb = 0.5;
+
+        hdrl_elemop_pow(a, ea, ARRAY_LEN(a), &b, &eb, 1, NULL);
+
+        cpl_test_rel(a[0], 0.25, HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], 0.15209233492346647, HDRL_EPS_ERROR * 10);
+        cpl_test_rel(a[1], 0.1111111111111111, HDRL_EPS_DATA);
+        cpl_test_rel(ea[1], 0.09597978726560344, HDRL_EPS_ERROR * 10);
+    }
+
+    /* masked test */
+    {
+        hdrl_data_t a[] = {2, 3};
+        hdrl_error_t ea[] = {0.5, 1};
+        const hdrl_data_t b[] = {-2, 6};
+        const hdrl_error_t eb[] = {0.5, 3};
+        cpl_binary mask[] = {0, 1};
+
+        hdrl_elemop_pow(a, ea, ARRAY_LEN(a), b, eb, ARRAY_LEN(b), mask);
+
+        cpl_test_rel(a[0], 0.25, HDRL_EPS_DATA);
+        cpl_test_rel(ea[0], 0.15209233492346647, HDRL_EPS_ERROR * 10);
+        cpl_test_rel(a[1], 3., HDRL_EPS_DATA);
+        cpl_test_rel(ea[1], 1., HDRL_EPS_ERROR * 10);
+    }
+
+    return cpl_error_get_code();
+}
+
 static cpl_error_code
 hdrl_test_image(void)
 {
@@ -466,6 +714,22 @@ hdrl_test_image(void)
     hdrl_elemop_image_div_image(iad, iae, ibd, ibe);
     cpl_test_rel(ad[0], ((5. - 3.) * 3.) / 3., HDRL_EPS_DATA);
 
+    hdrl_elemop_image_pow_image(iad, iae, ibd, ibe);
+    cpl_test_rel(ad[0], pow(((5. - 3.) * 3.) / 3., 3.), HDRL_EPS_DATA);
+
+    bd[0] = -1.;
+    ad[0] = 0.;
+    hdrl_elemop_image_pow_image(iad, iae, ibd, ibe);
+    cpl_test(cpl_image_is_rejected(iad, 1, 1));
+    cpl_image_accept_all(iad);
+    cpl_image_accept_all(iae);
+    ad[0] = 2.;
+
+    bd[0] = 0.;
+    hdrl_elemop_image_div_image(iad, iae, ibd, ibe);
+    cpl_test(cpl_image_is_rejected(iad, 1, 1));
+    bd[0] = 3.;
+
     /* test a has bpm */
     ad[0] = 2.;
     ae[0] = 0.5;
@@ -534,9 +798,17 @@ hdrl_test_image_scalar(void)
     hdrl_elemop_image_div_scalar(iad, iae, bd, be);
     cpl_test_rel(ad, ((5. - 3.) * 3.) / 3., HDRL_EPS_DATA);
 
+    hdrl_elemop_image_pow_scalar(iad, iae, bd, be);
+    cpl_test_rel(ad, pow(((5. - 3.) * 3.) / 3., 3.), HDRL_EPS_DATA);
+
     hdrl_elemop_image_div_scalar(iad, iae, 0., be);
     cpl_test(cpl_image_is_rejected(iad, 1, 1));
 
+    cpl_image_accept_all(iad);
+    cpl_image_set(iad, 1, 1, 0.);
+    hdrl_elemop_image_pow_scalar(iad, iae, -1., be);
+    cpl_test(cpl_image_is_rejected(iad, 1, 1));
+
     cpl_image_set(iad, 1, 1, 2.);
     cpl_image_reject(iad, 1, 1);
     hdrl_elemop_image_div_scalar(iad, iae, bd, be);
@@ -585,6 +857,9 @@ hdrl_test_imagelist(void)
     hdrl_elemop_imagelist_div_imagelist(iad, iae, ibd, ibe);
     cpl_test_rel(ad, ((5. - 3.) * 3.) / 3., HDRL_EPS_DATA);
 
+    hdrl_elemop_imagelist_pow_imagelist(iad, iae, ibd, ibe);
+    cpl_test_rel(ad, pow(((5. - 3.) * 3.) / 3., 3.), HDRL_EPS_DATA);
+
     cpl_image_set(_iad, 1, 1, 2.);
     cpl_image_reject(_iad, 1, 1);
     hdrl_elemop_imagelist_div_imagelist(iad, iae, ibd, ibe);
@@ -633,6 +908,9 @@ hdrl_test_imagelist_image(void)
     hdrl_elemop_imagelist_div_image(iad, iae, ibd, ibe);
     cpl_test_rel(ad, ((5. - 3.) * 3.) / 3., HDRL_EPS_DATA);
 
+    hdrl_elemop_imagelist_pow_image(iad, iae, ibd, ibe);
+    cpl_test_rel(ad, pow(((5. - 3.) * 3.) / 3., 3.), HDRL_EPS_DATA);
+
     cpl_image_set(_iad, 1, 1, 2.);
     cpl_image_reject(_iad, 1, 1);
     hdrl_elemop_imagelist_div_image(iad, iae, ibd, ibe);
@@ -662,6 +940,7 @@ int main(void)
     hdrl_test_sub();
     hdrl_test_mul();
     hdrl_test_div();
+    hdrl_test_pow();
     hdrl_test_image();
     hdrl_test_image_scalar();
     hdrl_test_imagelist();
