@@ -304,7 +304,6 @@ static int fors_sumflux(cpl_parameterlist *parlist, cpl_frameset *frameset)
 
     cpl_image        *master_bias = NULL;
     cpl_image        *exposure    = NULL;
-    cpl_image        *background  = NULL;
     cpl_image        *dummy       = NULL;
 
     cpl_table        *overscans   = NULL;
@@ -368,7 +367,7 @@ static int fors_sumflux(cpl_parameterlist *parlist, cpl_frameset *frameset)
     cpl_msg_indent_more();
 
     if (!dfs_equal_keyword(frameset, "ESO DET CHIP1 ID")) 
-        fors_sumflux_exit("Input frames are not from the same chip");
+        cpl_msg_warning(cpl_func,"Input frames are not from the same chip");
 
     nframes = cpl_frameset_count_tags(frameset, arc_tag)
             + cpl_frameset_count_tags(frameset, flat_tag);
